#ifndef LOCATION_H
#define LOCATION_H

#include "location_p.h"

#include <QString>
#include <QObject>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QGeoPositionInfo>
#include <math.h>

QTM_USE_NAMESPACE

class Location : public QObject
{
    Q_OBJECT

    Q_PROPERTY(QString longitude READ longitude())
    Q_PROPERTY(QString latitude READ latitude())
    Q_PROPERTY(QString address READ address() WRITE setAddress())
    Q_PROPERTY(QString label READ label() WRITE setLabel())
    Q_PROPERTY(bool valid READ isValid() WRITE setValid())

    public:
    Location(const QString &longitude, const QString &latitude, const QString &label=QString());
    Location(const QGeoPositionInfo &positionInfo, const QString &label=QString());
    Location(const Location &from);
    Location &operator=(const Location &from);
    Location(const QString &label=QString());

    ~Location();

    QString longitude() const;

    QString latitude() const;

    void setLocation(const QGeoPositionInfo &positionInfo);

    void setAddress(const QString &address) const;
    QString address() const;

    void setLabel(const QString &label) const;
    QString label() const;

    void setValid(bool valid) const;
    bool isValid() const;

    public Q_SLOTS:
    void resolveAddress(const QString &address);

    Q_SIGNALS:
    void becomeValid();
    void becomeInValid();
    void busy(bool busy);

    private Q_SLOTS:
    void replyFinished(QNetworkReply * reply);

    private:

    LocationPrivate *q;
    QNetworkAccessManager *manager;
};

#endif // LOCATION_H
