#include <stdio.h>
#include "location_p.h"

#include "xmlparser.h"

#include <QXmlStreamReader>
#include <QByteArray>
#include <QDebug>
#include <QMaemo5InformationBox>

LocationPrivate::LocationPrivate(const QString &longitude, const QString &latitude, const QString &label) :
    m_label(label),
    m_address(),
    m_longitude(longitude),
    m_latitude(latitude),
    m_valid(true)
{
}

LocationPrivate::LocationPrivate(const QString &label) :
    m_label(label),
    m_address(),
    m_longitude(QString()),
    m_latitude(QString()),
    m_valid(false)
{
}

LocationPrivate::~LocationPrivate()
{
    m_label="deleted";
    m_address="";
    m_longitude="";
    m_latitude="";
    m_valid=false;
}

void LocationPrivate::parseReply(const QByteArray &reply)
{
    qDebug() << "parsing/" << reply;

    XmlParser parser;
    QPair<QString,QString> parsedReply = parser.parseGeocode(reply);

    m_longitude = parsedReply.first;
    m_latitude  = parsedReply.second;

    qDebug() << "DONE parsing";

    if (parser.error()) {
        QMaemo5InformationBox::information(0, "address resolution error - please check address");
        m_valid = false;
    } else {
        qDebug() << "(" << m_longitude << "," << m_latitude << ")";
        if (m_longitude.isEmpty() ||  m_latitude.isEmpty()) {
            qDebug() << "is NOT valid";
            m_valid = false;
        } else {
            qDebug() << "is now valid";
            m_valid = true;
        }
    }
}

void LocationPrivate::setLabel(const QString &label)
{
    m_label = label;
}

QString LocationPrivate::label() const
{
    return m_label;
}

void LocationPrivate::setAddress(const QString &address)
{
    m_address = address;
}

QString LocationPrivate::address() const
{
    return m_address;
}

void LocationPrivate::setLongitude(double longitude)
{
    m_longitude = QString("%1").arg(longitude);
}

void LocationPrivate::setLongitude(const QString &longitude)
{
    m_longitude = longitude;
}

QString LocationPrivate::longitude() const
{
    return m_longitude;
}

void LocationPrivate::setlatitude(double latitude)
{
    m_latitude = QString("%1").arg(latitude);
}

void LocationPrivate::setlatitude(const QString &latitude)
{
    m_latitude = latitude;
}

QString LocationPrivate::latitude() const
{
    return m_latitude;
}

void LocationPrivate::setValid(bool valid)
{
    qDebug() << Q_FUNC_INFO << "/valid/" << valid;
    m_valid = valid;
}

bool LocationPrivate::isValid() const
{
    return m_valid;
}
